const OpenAiAcessor = require('../services/OpenAiAcessor');
const MockGenerator = require('../mocks/MockGenerator');
const mockConfig = require('../mocks/json/config');
const testCases = require('./openai-resolver-test-cases.json');

describe('OpenAi acessor testing...', () => {
  const acessor = new OpenAiAcessor(mockConfig.OPEN_AI_URL, true);

  describe.each(Object.keys(testCases))('Filter response %s', lang => {
    const langTestCases = testCases[lang];

    it.each(langTestCases.shouldBlock)('%# Blocked', (companyId, question, response) => {
      console.log(question);
      console.log(response);
      expect(acessor.filterResponse(response, lang)).toEqual(true);
    });

    it.each(langTestCases.shouldPass)('%# Allowed', (companyId, question, response) => {
      console.log(question);
      console.log(response);
      expect(acessor.filterResponse(response, lang)).toEqual(false);
    });
  });

  describe('Filter response other cases', () => {
    it('Pass with another language', () => {
      expect(acessor.filterResponse('O conteúdo é uma mensagem', 'jp-jp')).toEqual(false);
    });

    it('Fallback reason', async () => {
      const data = MockGenerator.generateIncoming('Bom dia, gostaria de tirar uma dúvida');
      data.language = 'pt-br';
      jest.spyOn(OpenAiAcessor.prototype, 'requestData').mockImplementationOnce(() =>
        Promise.resolve({
          body: {
            choices: [
              {
                message: {
                  content:
                    'O conteúdo é uma mensagem de um assistente virtual de resorts no Brasil.',
                },
              },
            ],
          },
        }),
      );

      const result = await acessor.resolveText(data);
      expect(result.processingStep.fallbackReason).toBe('blocked_by_filter');
    });
  });
});
