const { CacheRedis } = require('../util/CacheRedis');
const md5 = require('md5');
const AsksuiteProcessingNaturalUtils = require('../AsksuiteProcessingNaturalUtils');
const AsksuiteTranslator = require('../util/Translator');
const defaultFallbackIntent = require('../util/defaultFallbackIntent');

class ExactMatcherAccessor {
  constructor(config) {
    this.config = config.config;
    this.client = config.redis;
    this.caches = {};
    this.utils = new AsksuiteProcessingNaturalUtils();

    this.caches = new CacheRedis({
      client: this.client,
      prefixKey: `exact_matcher`,
      expirationTime: 0,
      isJson: false,
    });
  }

  async resolveText(request) {
    const { companyId, language: requestLanguage, languages, defaultLanguage } = request;
    const language = this.utils.resolveLanguage(requestLanguage, languages);
    let { text } = request;

    if (defaultLanguage && language.substr(0, 2) !== defaultLanguage.substr(0, 2)) {
      const result = await this.resolveText({
        ...request,
        defaultLanguage: language,
      });

      if (result.intent !== 'naoentendi') {
        return result;
      }

      const asksuiteTranslator = new AsksuiteTranslator(
        this.config.INFOCHAT_DATA_SERVER,
        this.config.GOOGLE_TRANSLATE_KEY,
      );

      const translation = await asksuiteTranslator.googleTranslateIfNeeded(
        text,
        language,
        languages,
        defaultLanguage,
      );

      text = translation.translation;
    }

    const textHash = md5(this.normalizeText(text));
    const key = `${companyId}:${defaultLanguage.substr(0, 2)}:${textHash}`;

    const intentFound = await this.caches.get(key);
    const dialogFound = this.utils.findDialogByIntent(request, intentFound);
    let dialog = '';
    if (dialogFound && dialogFound.dialog) {
      dialog = dialogFound.dialog;
    }

    return {
      intent:
        intentFound ||
        (request.findIntent && request.findIntent.intent) ||
        defaultFallbackIntent(companyId).intent,
      dialog,
      fromCache: true,
      extractedData: null,
    };
  }

  normalizeText(text) {
    try {
      return text
        .trim()
        .replace(/\s\s+/g, ' ')
        .replace(/[.,\/#!$%^&*;:{}=\-_`~()?@]/g, '')
        .normalize('NFD')
        .replace(/[\u0300-\u036f]/g, '')
        .toLowerCase();
    } catch (e) {
      console.error('RedisService.normalizeText', text, e);
      return text;
    }
  }
}

module.exports = ExactMatcherAccessor;
