const Util = require('../util');

class KeyMatcherAccessor {
  constructor(awsLambdaCaller, openAiEnabled = false) {
    this.awsLambdaCaller = awsLambdaCaller;
    this.openAiEnabled = openAiEnabled;
  }

  resolveText(request) {
    let intent = null;

    return new Promise(resolve => {
      const executor = async () => {
        try {
          // KeyMatcher
          intent = await this.awsLambdaCaller.call(request, 'asksuite-NLP-dev-keyMatcherNLP');
          if (!intent || !intent.intent) {
            throw new Error('Intent not present in response: \n' + JSON.stringify(intent));
          }

          if (this.openAiEnabled && !Util.isQuote(intent.intent)) {
            const processingStep = intent.processingStep || {
              text: request.text,
              intentFound: intent.intent,
            };

            intent = {
              ...request.findIntent,
              processingStep: {
                language: request.language,
                resolver: 'KEY_MATCHER',
                processingStep: { ...processingStep, fallbackReason: 'openai_enabled' },
              },
            };
          }
        } catch (e) {
          intent = request.findIntent;
          console.log('Erro no keymatcher ', e);
        }
        resolve(intent);
      };

      executor();
    });
  }
}

module.exports = KeyMatcherAccessor;
